<?php
/**
 * @file
 * Provide a gantt chart style plugin for Views.
 */

class ganttview_style_plugin extends views_plugin_style {
  /**
   * Set default options
   */
  function option_definition() {
    $options = parent::option_definition();
    $options['tasktitle'] = array('default' => '');
    $options['startdate'] = array('default' => '');
    $options['enddate'] = array('default' => '');
    $options['parentid'] = array('default' => '');
    $options['parenttitle'] = array('default' => '');
	  $options['parentend'] = array('default' => '');
    $options['parentstart'] = array('default' => '');
	  $options['parenttype'] = array('default' => '');
    $options['tasktype'] = array('default' => '');

    return $options;
  }

  /**
   * Provide form
   */
  function options_form(&$form, &$form_state) {
    $optional = array('' => t('<None>'));
    $options = $this->display->handler->get_field_labels();
    $optional += $options;
    $form['tasktitle'] = array(
      '#title' => t('Task title field'),
      '#description' => t('Pick the field that defines task titles.'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->options['tasktitle'],
    );
    $form['startdate'] = array(
      '#title' => t('Start date field'),
      '#description' => t('Pick the date field that defines start dates.'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->options['startdate'],
    );
    $form['enddate'] = array(
      '#title' => t('End date field'),
      '#description' => t('Pick the field that defines end dates. Note that if this field is empty, the task will appear as a milestone.'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->options['enddate'],
    );
    $form['tasktype'] = array(
      '#title' => t('Task Content Type'),
      '#description' => t('Pick the field that defines your "Task" content type.'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->options['tasktype'],
    );
    $form['parentid'] = array(
      '#title' => t('Parent ID field'),
      '#description' => t('Pick the field that defines the nid of parents of tasks.'),
      '#type' => 'select',
      '#options' => $optional,
      '#default_value' => $this->options['parentid'],
    );
    $form['parenttitle'] = array(
      '#title' => t('Parent title field'),
      '#description' => t('Pick the field that defines the title of parents of tasks.'),
      '#type' => 'select',
      '#options' => $optional,
      '#default_value' => $this->options['parenttitle'],
    );
    $form['parentstart'] = array(
      '#title' => t('Parent Start Date'),
      '#description' => t('Pick the field that defines the start date of the parent.'),
      '#type' => 'select',
      '#options' => $optional,
      '#default_value' => $this->options['parentstart'],
    );
    $form['parentend'] = array(
      '#title' => t('Parent End Date'),
      '#description' => t('Pick the field that defines the end date of the parent.'),
      '#type' => 'select',
      '#options' => $optional,
      '#default_value' => $this->options['parentend'],
    );
    $form['parenttype'] = array(
      '#title' => t('Parent Content Type'),
      '#description' => t('Pick the field that defines your "Parent" content type.'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->options['parenttype'],
    );
  }

  function pre_render($result){
	  // Set our field names so we can retrieve them later.
	  $this->view->options['tasktitle'] = $this->view->field[$this->view->style_options['tasktitle']]->field_alias;
    $this->view->options['startdate'] = $this->view->field[$this->view->style_options['startdate']]->field;
    $this->view->options['enddate'] = $this->view->field[$this->view->style_options['enddate']]->field;
	  $this->view->options['parentstart'] = $this->view->field[$this->view->style_options['parentstart']]->field;
    $this->view->options['parentend'] = $this->view->field[$this->view->style_options['parentend']]->field;
    $this->view->options['parentid'] = $this->view->field[$this->view->style_options['parentid']]->field_alias;
    $this->view->options['parenttitle'] = $this->view->field[$this->view->style_options['parenttitle']]->field_alias;
    // Get the field names of the types of tasks and parents.
    $this->view->options['parenttype'] = $this->view->field[$this->view->style_options['parenttype']]->field_alias;
    $this->view->options['tasktype'] = $this->view->field[$this->view->style_options['tasktype']]->field_alias;
    // Reset the options as the actual values.
    $this->view->options['tasktype'] = $result[0]->{$this->view->options['tasktype']};
    $this->view->options['parenttype'] = $result[0]->{$this->view->options['parenttype']};
  }

  function render() {
    // Libraries for handling drag/drop/resize
    drupal_add_library('system', 'ui.resizable');
    drupal_add_library('system', 'ui.draggable');
    
    // For handling our date arguents.
    drupal_add_js(drupal_get_path('module', 'ganttview') . '/date.js');

    // Main script for generating the gantt chart.
    drupal_add_js(drupal_get_path('module', 'ganttview') . '/ganttview.js');

    // Main css for styling.
    drupal_add_css(drupal_get_path('module', 'ganttview') . '/ganttview.css');

    if (isset($this->view->live_preview) && $this->view->live_preview) {
      return t('ganttView not compatible with live preview.');
    }
    if (parent::uses_fields()){
      $output = parent::render();
    }
    else {
      drupal_set_message('ganttView requires Fields as row style', 'error');
    }	
    return $output;
  }
}

